# can pass frame as error call in `new_error()`

    Code
      f()
    Condition
      Error:
      ! ! my message
    Code
      g()
    Condition
      Error:
      ! ! my message

# can pass frame as error call in `throw()`

    Code
      f()
    Condition
      Error:
      ! ! my message
    Code
      g()
    Condition
      Error:
      ! ! my message

