# is_string

    Code
      assert_that(is_string(n))
    Condition
      Error:
      ! ! n is not a string (length 1 character)

---

    Code
      assert_that(is_string(n))
    Condition
      Error:
      ! ! n is not a string (length 1 character)

---

    Code
      assert_that(is_string(n))
    Condition
      Error:
      ! ! n is not a string (length 1 character)

---

    Code
      assert_that(is_string(n))
    Condition
      Error:
      ! ! n is not a string (length 1 character)

---

    Code
      assert_that(is_string(n))
    Condition
      Error:
      ! ! n is not a string (length 1 character)

---

    Code
      assert_that(is_string(n))
    Condition
      Error:
      ! ! n is not a string (length 1 character)

---

    Code
      assert_that(is_string(n))
    Condition
      Error:
      ! ! n is not a string (length 1 character)

# is_string_or_null

    Code
      assert_that(is_string_or_null(n))
    Condition
      Error:
      ! ! n must be a string (length 1 character) or NULL

---

    Code
      assert_that(is_string_or_null(n))
    Condition
      Error:
      ! ! n must be a string (length 1 character) or NULL

---

    Code
      assert_that(is_string_or_null(n))
    Condition
      Error:
      ! ! n must be a string (length 1 character) or NULL

---

    Code
      assert_that(is_string_or_null(n))
    Condition
      Error:
      ! ! n must be a string (length 1 character) or NULL

---

    Code
      assert_that(is_string_or_null(n))
    Condition
      Error:
      ! ! n must be a string (length 1 character) or NULL

---

    Code
      assert_that(is_string_or_null(n))
    Condition
      Error:
      ! ! n must be a string (length 1 character) or NULL

# is_flag

    Code
      assert_that(is_flag(n))
    Condition
      Error:
      ! ! n is not a flag (length 1 logical)

---

    Code
      assert_that(is_flag(n))
    Condition
      Error:
      ! ! n is not a flag (length 1 logical)

---

    Code
      assert_that(is_flag(n))
    Condition
      Error:
      ! ! n is not a flag (length 1 logical)

---

    Code
      assert_that(is_flag(n))
    Condition
      Error:
      ! ! n is not a flag (length 1 logical)

---

    Code
      assert_that(is_flag(n))
    Condition
      Error:
      ! ! n is not a flag (length 1 logical)

---

    Code
      assert_that(is_flag(n))
    Condition
      Error:
      ! ! n is not a flag (length 1 logical)

---

    Code
      assert_that(is_flag(n))
    Condition
      Error:
      ! ! n is not a flag (length 1 logical)

# is_integerish_scalar

    Code
      assert_that(is_integerish_scalar(n))
    Condition
      Error:
      ! ! n is not a length 1 integer

---

    Code
      assert_that(is_integerish_scalar(n))
    Condition
      Error:
      ! ! n is not a length 1 integer

---

    Code
      assert_that(is_integerish_scalar(n))
    Condition
      Error:
      ! ! n is not a length 1 integer

---

    Code
      assert_that(is_integerish_scalar(n))
    Condition
      Error:
      ! ! n is not a length 1 integer

---

    Code
      assert_that(is_integerish_scalar(n))
    Condition
      Error:
      ! ! n is not a length 1 integer

---

    Code
      assert_that(is_integerish_scalar(n))
    Condition
      Error:
      ! ! n is not a length 1 integer

---

    Code
      assert_that(is_integerish_scalar(n))
    Condition
      Error:
      ! ! n is not a length 1 integer

---

    Code
      assert_that(is_integerish_scalar(n))
    Condition
      Error:
      ! ! n is not a length 1 integer

# is_pid

    Code
      assert_that(is_pid(n))
    Condition
      Error:
      ! ! n is not a process id (length 1 integer)

---

    Code
      assert_that(is_pid(n))
    Condition
      Error:
      ! ! n is not a process id (length 1 integer)

---

    Code
      assert_that(is_pid(n))
    Condition
      Error:
      ! ! n is not a process id (length 1 integer)

---

    Code
      assert_that(is_pid(n))
    Condition
      Error:
      ! ! n is not a process id (length 1 integer)

---

    Code
      assert_that(is_pid(n))
    Condition
      Error:
      ! ! n is not a process id (length 1 integer)

---

    Code
      assert_that(is_pid(n))
    Condition
      Error:
      ! ! n is not a process id (length 1 integer)

---

    Code
      assert_that(is_pid(n))
    Condition
      Error:
      ! ! n is not a process id (length 1 integer)

---

    Code
      assert_that(is_pid(n))
    Condition
      Error:
      ! ! n is not a process id (length 1 integer)

# is_flag_or_string

    Code
      assert_that(is_flag_or_string(n))
    Condition
      Error:
      ! ! n is not a flag or a string

---

    Code
      assert_that(is_flag_or_string(n))
    Condition
      Error:
      ! ! n is not a flag or a string

---

    Code
      assert_that(is_flag_or_string(n))
    Condition
      Error:
      ! ! n is not a flag or a string

---

    Code
      assert_that(is_flag_or_string(n))
    Condition
      Error:
      ! ! n is not a flag or a string

---

    Code
      assert_that(is_flag_or_string(n))
    Condition
      Error:
      ! ! n is not a flag or a string

---

    Code
      assert_that(is_flag_or_string(n))
    Condition
      Error:
      ! ! n is not a flag or a string

---

    Code
      assert_that(is_flag_or_string(n))
    Condition
      Error:
      ! ! n is not a flag or a string

# is_existing_file

    Code
      assert_that(is_existing_file(tempfile()))
    Condition
      Error:
      ! ! File tempfile() does not exist

