# errors

    Code
      conn_create_fifo(read = TRUE, write = TRUE)
    Condition
      Error:
      ! ! Bi-directional FIFOs are not supported currently

---

    Code
      conn_connect_fifo(read = TRUE, write = TRUE)
    Condition
      Error:
      ! ! Bi-directional FIFOs are not supported currently

